#import "PSFPController.h"

#define TableRowType @"row"
#define TableRowTypes [NSArray arrayWithObjects:@"row",nil]

@implementation PSFPController

- (id)init
{
	[super init];
	player = [[PSFPlayer alloc] initWithDelegate: self];
	playListArray = [[NSMutableArray alloc] init];
	playListStatusArray = [[NSMutableArray alloc] init];
	playListConvertArray = [[NSMutableArray alloc] init];
	playListCheckBoxArray = [[NSMutableArray alloc] init];
	nowPlaying = NO;
	nowConverting = NO;
	launched = NO;
	return self;
}

- (IBAction)play:(id)sender
{
	if(!nowPlaying) {
		if([playListArray count]) {
			[self playFileAtIndex: 0];
			nowPlaying = YES;
		}
		else return;
	}
	else {
		if([player isPause]) [playButton setImage:[NSImage imageNamed:@"pause"]];
		else [playButton setImage:[NSImage imageNamed:@"play"]];
		[player pause];
	}
}

- (IBAction)stop:(id)sender
{
	if(nowPlaying) {
		[player stop];
		nowPlaying = NO;
		[playingTime setStringValue:@""];
		[statusText setStringValue: @""];
		[currentFile setStringValue: @""];
		if(playingFile) [playingFile release];
		[self clearStatus];
		[playListTable reloadData];
		[convertMenu setEnabled: YES];
		[playButton setImage:[NSImage imageNamed:@"play"]];
		nowPlaying = NO;
	}
	else if(nowConverting) {
		int index = [playListArray indexOfObjectIdenticalTo: [converter currentFile]];
		[playListStatusArray replaceObjectAtIndex: index withObject: [NSNumber numberWithInt: 2]];
		[converter stop];
	}
}

- (IBAction)onDoubleClick:(id)sender
{
	if(nowConverting) return;
	if([sender clickedRow] < 0) return;
	if(nowPlaying) {
		int index = [playListArray indexOfObjectIdenticalTo: playingFile];
		[playListStatusArray replaceObjectAtIndex: index withObject: [NSNumber numberWithInt: 0]];
	}
	[self playFileAtIndex: [sender clickedRow]];
	nowPlaying = YES;
}

- (IBAction)skip:(id)sender
{
	if(!nowPlaying) return;
	int index = [playListArray indexOfObjectIdenticalTo: playingFile];
	[playListStatusArray replaceObjectAtIndex: index withObject: [NSNumber numberWithInt: 0]];
	if(index >= [playListArray count]-1) index = -1;
	[self playFileAtIndex: index+1];
	[playListTable reloadData];
}

- (IBAction)reverse:(id)sender
{
	if(!nowPlaying) return;
	int index = [playListArray indexOfObjectIdenticalTo: playingFile];
	[playListStatusArray replaceObjectAtIndex: index withObject: [NSNumber numberWithInt: 0]];
	if(index == 0) index = [playListArray count];
	[self playFileAtIndex: index-1];
	[playListTable reloadData];
}

- (void)playFileAtIndex:(int)index
{
	[convertMenu setEnabled: NO];
	[playButton setImage:[NSImage imageNamed:@"pause"]];
	[player playPSF: [playListArray objectAtIndex: index]];
	[self clearStatus];
	if(nowPlaying) [playingFile release];
	playingFile = [[playListArray objectAtIndex: index] retain];
	[statusText setStringValue: @"Playing"];
	[playingTime setStringValue:@""];
	[currentFile setStringValue: [[playListArray objectAtIndex: index] lastPathComponent]];
	[playListStatusArray replaceObjectAtIndex: index withObject: [NSNumber numberWithInt: 3]];
	[playListTable reloadData];
}

- (void)playDone
{
	if(nowPlaying) {
		int index = [playListArray indexOfObjectIdenticalTo: playingFile];
		[playListStatusArray replaceObjectAtIndex: index withObject: [NSNumber numberWithInt: 0]];
		if(index >= [playListArray count]-1) index = -1;
		[self playFileAtIndex: index+1];
		[playListTable reloadData];
	}
}

- (void)clearStatus
{
	int n = [playListArray count];
	int i;
	[playListStatusArray removeAllObjects];
	for(i=0;i<n;i++) {
		[playListStatusArray addObject: [NSNumber numberWithInt: 0]];
	}
}

- (IBAction)convert:(id)sender
{
	int i,n;
	n = [playListArray count];
	NSMutableArray *ar = [NSMutableArray arrayWithCapacity: 0];
	for(i=0;i<n;i++) {
		if([playListTable isRowSelected:i]) {
			[ar addObject: [playListArray objectAtIndex: i]];
		}
	}
	if(![ar count]) return;
	[self clearStatus];
	[playButton setEnabled: NO];
	[skipButton setEnabled: NO];
	[reverseButton setEnabled: NO];
	[convertMenu setEnabled: NO];
	converter = [[PSFPConverter alloc] initWithDelegate: self];
	if([[dupRadio selectedCell] tag] == 0) [converter setAllowDuplicatedName: NO];
	else [converter setAllowDuplicatedName: YES];
	if([[outputRadio selectedCell] tag] == 1) [converter setOutputDirectory: [fixedOutputDir stringValue]];
	[converter setConvertFiles: ar];
	[converter convert];
	[statusText setStringValue: @"Converting"];
	[currentFile setStringValue: [[converter currentFile] lastPathComponent]];
	int index = [playListArray indexOfObjectIdenticalTo: [converter currentFile]];
	[playListStatusArray replaceObjectAtIndex: index withObject: [NSNumber numberWithInt: 3]];
	[playListTable reloadData];
	nowConverting = YES;
}

- (void)convertDone:(NSString *)doneFile
{
	NSString *file = [converter currentFile];
	[currentFile setStringValue: [file lastPathComponent]];
	int index = [playListArray indexOfObjectIdenticalTo: doneFile];
	[playListStatusArray replaceObjectAtIndex: index withObject: [NSNumber numberWithInt: 1]];
	index = [playListArray indexOfObjectIdenticalTo: file];
	[playListStatusArray replaceObjectAtIndex: index withObject: [NSNumber numberWithInt: 3]];
	[playListTable reloadData];
}

- (void)convertComplete:(NSString *)doneFile
{
	int index = [playListArray indexOfObjectIdenticalTo: doneFile];
	[playListStatusArray replaceObjectAtIndex: index withObject: [NSNumber numberWithInt: 1]];
	[playingTime setStringValue:@""];
	[statusText setStringValue: @""];
	[currentFile setStringValue: @""];
	[converter release];
	[playListTable reloadData];
	[playButton setEnabled: YES];
	[skipButton setEnabled: YES];
	[reverseButton setEnabled: YES];
	[convertMenu setEnabled: YES];
	nowConverting = NO;
}

- (void)convertTerminated
{
	nowConverting = NO;
	[playingTime setStringValue:@""];
	[statusText setStringValue: @""];
	[currentFile setStringValue: @""];
	[playListTable reloadData];
	[playButton setEnabled: YES];
	[skipButton setEnabled: YES];
	[reverseButton setEnabled: YES];
	[converter release];
	[convertMenu setEnabled: YES];
}

- (void)setPlayTime:(NSString *) time
{
	[playingTime setStringValue:time];
}

- (IBAction)prefCancel:(id)sender
{
	[prefPane close];
	[self loadPrefs];
}

- (IBAction)prefOK:(id)sender
{
	[prefPane close];
	[self savePrefs];
}

- (IBAction)showPref:(id)sender
{
	[prefPane center];
	[prefPane makeKeyAndOrderFront:self];
}

- (IBAction)enableOutputDir:(id)sender
{
	if([[sender selectedCell] tag] == 1) {
		[fixedOutputDir setEnabled: YES];
		[setOutputDirButton setEnabled:YES];
	}
	else {
		[fixedOutputDir setEnabled: NO];
		[setOutputDirButton setEnabled:NO];
	}
}

- (IBAction)setOutputDir:(id)sender
{
	NSOpenPanel *op;
	int stat;
	
	op = [ NSOpenPanel openPanel ];
	
	//[ op setTitle : NSLocalizedString(@"selectdir",@"") ];
	[ op setCanChooseDirectories : YES ];
	[ op setCanChooseFiles : NO ];
	
	stat = [ op runModalForDirectory : nil
								file : nil
							   types : nil ];
							   
	if(stat != NSOKButton) return;
	
	[fixedOutputDir setStringValue:[op filename]];
}

- (IBAction)toggleUnlimited:(id)sender
{
	if([sender state] == NSOnState) {
		[depthStepper setEnabled: NO];
		[subdirectoryDepth setTextColor: [NSColor lightGrayColor]];
	}
	else {
		[depthStepper setEnabled: YES];
		[subdirectoryDepth setTextColor: [NSColor blackColor]];
	}
}

- (IBAction)setWindowOpacity:(id)sender
{
	[mainWindow setAlphaValue: [sender floatValue]];
}

- (IBAction)toggleWindowAppearance:(id)sender
{
	if([sender state] == NSOnState) {
		[mainWindow _setTexturedBackground: YES];
	}
	else {
		[mainWindow _setTexturedBackground: NO];
	}
}

- (IBAction)setWindowLevel:(id)sender
{
	if([[sender selectedItem] tag] == 0) {
		[mainWindow setLevel: NSNormalWindowLevel];
	}
	else {
		[mainWindow setLevel: NSFloatingWindowLevel];
	}
}

- (void)savePrefs
{
	NSUserDefaults *pref = [NSUserDefaults standardUserDefaults];
	[pref setObject:[fixedOutputDir stringValue] forKey:@"OutputDir"];
	[pref setInteger:[[dupRadio selectedCell] tag] forKey:@"DupRadio"];
	[pref setInteger:[[outputRadio selectedCell] tag] forKey:@"OutputRadio"];
	if([autoSaveFlag state] == NSOnState) [pref setObject:playListArray forKey:@"PlayList"];
	else [pref removeObjectForKey: @"PlayList"];
	[pref setInteger: [autoSaveFlag state] forKey: @"AutoSave"];
	[pref setInteger: [subdirectoryDepth intValue] forKey : @"Depth"];
	[pref setInteger: [unlimitedCheck state] forKey: @"Unlimited"];
	[pref setFloat: [opacitySlider floatValue] forKey: @"Opacity"];
	[pref setInteger: [texturedCheck state] forKey: @"Texture"];
	[pref setInteger:[[windowLevelPopup selectedItem] tag] forKey:@"WindowLevel"];
	[pref synchronize];
}

- (void)loadPrefs
{
	NSUserDefaults *pref = [NSUserDefaults standardUserDefaults];
	id temp;
	int i;
	if(temp=[pref objectForKey:@"DupRadio"]) {
		i = [temp intValue];
		[dupRadio selectCellWithTag: i];
	}
	if(temp=[pref objectForKey:@"OutputRadio"]) {
		i = [temp intValue];
		[outputRadio selectCellWithTag: i];
		if(i==1) {
			[fixedOutputDir setEnabled: YES];
			[setOutputDirButton setEnabled:YES];
		}
		else {
			[fixedOutputDir setEnabled: NO];
			[setOutputDirButton setEnabled:NO];
		}
	}
	if(temp=[pref objectForKey:@"OutputDir"]) {
		[fixedOutputDir setStringValue:temp];
	}
	if(temp=[pref objectForKey:@"AutoSave"]) {
		[autoSaveFlag setState: [temp intValue]];
	}
	else [fixedOutputDir setStringValue:[@"~/Desktop" stringByExpandingTildeInPath]];
	if(temp=[pref objectForKey:@"Depth"]) {
		[subdirectoryDepth setIntValue: [temp intValue]];
		[depthStepper setIntValue: [temp intValue]];
	}
	if(temp=[pref objectForKey:@"Unlimited"]) {
		[unlimitedCheck setState: [temp intValue]];
		if([unlimitedCheck state] == NSOnState) {
			[depthStepper setEnabled: NO];
			[subdirectoryDepth setTextColor: [NSColor lightGrayColor]];
		}
		else {
			[depthStepper setEnabled: YES];
			[subdirectoryDepth setTextColor: [NSColor blackColor]];
		}
	}
	if(temp=[pref objectForKey:@"Opacity"]) {
		[mainWindow setAlphaValue: [temp floatValue]];
		[opacitySlider setFloatValue: [temp floatValue]];
	}
	if(temp=[pref objectForKey:@"Texture"]) {
		[texturedCheck setState: [temp intValue]];
		if([texturedCheck state] == NSOnState) {
			[mainWindow _setTexturedBackground: YES];
		}
		else {
			[mainWindow _setTexturedBackground: NO];
		}
	}
	if(temp=[pref objectForKey:@"WindowLevel"]) {
		[windowLevelPopup selectItemAtIndex: [windowLevelPopup indexOfItemWithTag:[temp intValue]]];
		if([temp intValue] == 0) {
			[mainWindow setLevel: NSNormalWindowLevel];
		}
		else {
			[mainWindow setLevel: NSFloatingWindowLevel];
		}
	}
	if(launched) return;
	if(temp=[pref objectForKey:@"PlayList"]) {
		[self appendFile:temp atIndex: 0 recursion: 0];
	}
}

- (void)removeSelectedFile:(id)table
{
	int n,m,i;
	
	if(![playListArray count]) return;
	if(nowConverting) return;
	
	n = [playListArray count];
	if(nowPlaying) m = [playListArray indexOfObjectIdenticalTo: playingFile];
	else m = -1;
	for(i=n-1;i>=0;i--) {
		if([playListTable isRowSelected:i]) {
			if(i == m) continue;
			[playListArray removeObjectAtIndex:i];
			[playListStatusArray removeObjectAtIndex:i];
			//if(i < [statusArray count]) [statusArray removeObjectAtIndex:i];
		}
	}
	[playListTable reloadData];
	
}

- (int)appendFile:(NSArray *)files atIndex: (int)index recursion: (int)r
{
	int i, j, added=0, ret, m;
	int n = [files count];
	FILE *fp;
	char buf[4];
	NSFileManager *fm = [NSFileManager defaultManager];
	BOOL isDir;
	NSString *file;
	/*
	for(i=0;i<n;i++) {
		NSLog(@"hoge");
		NSRect myRect = [playListTable frameOfCellAtColumn:0 row: m+i];
		NSProgressIndicator *myCheck = [[NSProgressIndicator alloc] initWithFrame: myRect];
		//[myCheck setButtonType: NSSwitchButton];
		[playListConvertArray addObject:[NSNumber numberWithInt: 1]];
		[playListCheckBoxArray addObject: myCheck];
		[playListTable addSubview: myCheck];
		[myCheck release];
	}
	*/
	for(i=0;i<n;i++) {
		file = [files objectAtIndex: i];
		[fm fileExistsAtPath:file isDirectory:&isDir];
		if(isDir) {
			if([unlimitedCheck state] == NSOffState && r >= [subdirectoryDepth intValue]) continue;
			NSArray *arr = [fm directoryContentsAtPath: file];
			NSMutableArray *arr2 = [NSMutableArray arrayWithArray: 0];
			m = [arr count];
			for(j=0;j<m;j++) {
				[arr2 addObject: [file stringByAppendingPathComponent: [arr objectAtIndex: j]]];
			}
			ret = [self appendFile: arr2 atIndex: index+added recursion: r+1];
			added = added + ret;
			continue;
		}
		
		if([[file pathExtension] caseInsensitiveCompare: @"psflib"] == NSOrderedSame) continue;
		
		if(!(fp=fopen([file UTF8String],"rb"))) {
			continue;
		}
		if(fread(buf,1,4,fp)!=4) {
			fclose(fp);
			continue;
		}
		fclose(fp);
		if(memcmp(buf,"PSF\x01",4))	// Only allow for PSF1 files for now.
		{
			continue;
		}
		[playListArray insertObject: file atIndex: index+added];
		[playListStatusArray insertObject: [NSNumber numberWithInt: 0] atIndex: index+added];
		added++;
	}
	[playListTable reloadData];
	return added;
}

/* delegate methods */

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
	return [playListArray count];
}

- (id)tableView:(NSTableView *)tableView
		objectValueForTableColumn:(NSTableColumn *)tableColumn
		row:(int)row
{
	id identifier = [tableColumn identifier];
	if([identifier isEqual:@"Status"])  {
		if([[playListStatusArray objectAtIndex:row] intValue] == 1) return [NSImage imageNamed:@"done"];
		else if([[playListStatusArray objectAtIndex:row] intValue] == 2) return [NSImage imageNamed:@"error"];
		else if([[playListStatusArray objectAtIndex:row] intValue] == 3) return [NSImage imageNamed:@"process"];
		else return nil;
	}
	else if([identifier isEqual:@"File"]){
		return [[playListArray objectAtIndex:row] lastPathComponent];
	}
	return nil;
}


-(BOOL)tableView:(NSTableView *)tv writeRows:(NSArray*)rows toPasteboard:(NSPasteboard*)pboard
{
	
	[pboard declareTypes:TableRowTypes owner:self];
	[pboard setPropertyList:rows forType:TableRowType];
	
	return YES;
}

-(NSDragOperation)tableView:(NSTableView*)tv validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)op
{
	if (op == NSTableViewDropAbove ) {
		return NSDragOperationGeneric;
	} else {
		return NSDragOperationNone;
	}
}

-(BOOL)tableView:(NSTableView*)tv acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)op
{

	NSPasteboard *pboard = [info draggingPasteboard];
	NSArray *ar = [pboard propertyListForType: NSFilenamesPboardType];
	if(ar) {
		NSArray *sortedArray = [ar sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
		[self appendFile:sortedArray atIndex: row recursion: 0];
		return YES;
	}
	
	NSEnumerator *e=[[pboard propertyListForType:TableRowType] objectEnumerator];
	NSNumber *number;
	NSMutableArray *upperArray1=[NSMutableArray arrayWithArray:[playListArray subarrayWithRange:NSMakeRange(0,row)]];
	NSMutableArray *lowerArray1=[NSMutableArray arrayWithArray:[playListArray subarrayWithRange:NSMakeRange(row,([playListArray count] - row))]];
	NSMutableArray *middleArray1=[NSMutableArray arrayWithCapacity:0];
	NSMutableArray *upperArray2=[NSMutableArray arrayWithArray:[playListStatusArray subarrayWithRange:NSMakeRange(0,row)]];
	NSMutableArray *lowerArray2=[NSMutableArray arrayWithArray:[playListStatusArray subarrayWithRange:NSMakeRange(row,([playListArray count] - row))]];
	NSMutableArray *middleArray2=[NSMutableArray arrayWithCapacity:0];
	id object1,object2;
	int i;

	if (op == NSTableViewDropAbove && [pboard availableTypeFromArray:TableRowTypes] != nil) {

		while ((number=[e nextObject]) != nil) {
			object1=[playListArray objectAtIndex:[number intValue]];
			object2=[playListStatusArray objectAtIndex:[number intValue]];
			[middleArray1 addObject:object1];
			[middleArray2 addObject:object2];
			i = [upperArray1 indexOfObjectIdenticalTo: object1];
			if(i != NSNotFound) {
				[upperArray1 removeObjectAtIndex: i];
				[upperArray2 removeObjectAtIndex: i];
			}
			i = [lowerArray1 indexOfObjectIdenticalTo: object1];
			if(i != NSNotFound) {
				[lowerArray1 removeObjectAtIndex: i];
				[lowerArray2 removeObjectAtIndex: i];
			}
		}

		[playListArray removeAllObjects];
		[playListStatusArray removeAllObjects];

		[playListArray addObjectsFromArray:upperArray1];
		[playListArray addObjectsFromArray:middleArray1];
		[playListArray addObjectsFromArray:lowerArray1];
		[playListStatusArray addObjectsFromArray:upperArray2];
		[playListStatusArray addObjectsFromArray:middleArray2];
		[playListStatusArray addObjectsFromArray:lowerArray2];

		[playListTable reloadData];
		[playListTable deselectAll:nil];
	
		for (i=[upperArray1 count];i<([upperArray1 count] + [middleArray1 count]);i++) {
			[playListTable selectRow:i byExtendingSelection:[playListTable allowsMultipleSelection]];
		}

		return YES;
	} else {
		return NO;
	}
	
	return YES;
}


- (void)applicationDidFinishLaunching: (NSNotification *)notification
{
	[self loadPrefs];
	NSArray* array = [NSArray arrayWithObjects:NSFilenamesPboardType, @"row", nil];
	[playListTable registerForDraggedTypes:array];
	[[playListTable tableColumnWithIdentifier:@"Status"] setDataCell:[[NSImageCell alloc]init]];
	[playListTable setDoubleAction: @selector(onDoubleClick:)];
	[mainWindow setFrameAutosaveName: @"MainWindow"];
	[mainWindow makeKeyAndOrderFront: self];
	BOOL canWeChangeTextureOnTheFly = [mainWindow respondsToSelector:@selector(_setTexturedBackground:)];
	if(!canWeChangeTextureOnTheFly) [texturedCheck setEnabled: NO];
	launched = YES;
}


- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
	if(nowConverting || nowPlaying) {
		int ret = NSRunAlertPanel(@"Quit", @"Some files are being processed. Quit anyway?",@"Cancel", nil, @"OK");
		if(ret == NSAlertDefaultReturn) return NO;
	}
	[self savePrefs];
	return YES;
}

- (BOOL)windowShouldClose:(id)sender
{
	[NSApp terminate:nil];
	return NO;
}



@end
