//
//  PSFPConverter.m
//  PSFPlay
//
//  Created by tmkk on Sat Jul 24 2004.
//  Copyright (c) 2004 tmkk. All rights reserved.
//

#import "PSFPConverter.h"
#import "PSFPController.h"

@implementation PSFPConverter

- (id)initWithDelegate: (id)del
{
	[super init];
	delegate = [del retain];
	currentIndex = 0;
	outputDirectory = nil;
	allowDuplicatedName = NO;
	isConverting = NO;
	return self;
}

- (void)dealloc
{
	[convertFiles release];
	[delegate release];
	[super dealloc];
}

- (void)setConvertFiles:(NSArray *)files
{
	currentIndex = 0;
	convertFiles = [files retain];
}

- (void)setOutputDirectory:(NSString *)dir
{
	if(!dir) outputDirectory = nil;
	else outputDirectory = [dir retain];
}

- (void)setAllowDuplicatedName: (BOOL)flag
{
	allowDuplicatedName = flag;
}

- (void)convert
{
	NSFileManager *fm = [NSFileManager defaultManager];
	int k = 1;
	NSString *outputFile;
	
	task = [[NSTask alloc] init];
	[task setStandardOutput: [NSPipe pipe]];
	[task setLaunchPath: [[NSBundle mainBundle] pathForResource:@"psfplay" ofType:nil inDirectory:nil]];
	
	if(outputDirectory) {
		outputFile = [[outputDirectory stringByAppendingPathComponent: [[[convertFiles objectAtIndex: currentIndex] lastPathComponent] stringByDeletingPathExtension]] stringByAppendingPathExtension: @"aiff"];
	}
	else
	{
		outputFile = [[[convertFiles objectAtIndex: currentIndex] stringByDeletingPathExtension] stringByAppendingPathExtension: @"aiff"];
	}
	
	if(!allowDuplicatedName) {
		if([fm fileExistsAtPath:outputFile]) {
			outputFile = [[outputFile stringByDeletingPathExtension] stringByAppendingPathExtension:[NSString stringWithFormat:@"%d.aiff",k]];
			while([fm fileExistsAtPath:outputFile]) {
				k++;
				outputFile = [outputFile stringByDeletingPathExtension];
				outputFile = [[outputFile stringByDeletingPathExtension] stringByAppendingPathExtension:[NSString stringWithFormat:@"%d.aiff",k]];
			}
		}
	}
	
	[task setArguments: [NSArray arrayWithObjects:@"-o", outputFile, [convertFiles objectAtIndex: currentIndex], nil]];
	[[NSNotificationCenter defaultCenter] addObserver:self 
        selector:@selector(getData:) 
        name: @"NSFileHandleReadCompletionNotification"
        object: [[task standardOutput] fileHandleForReading]];
	[[NSNotificationCenter defaultCenter] addObserver:self 
        selector:@selector(taskDidEnd:) 
        name: @"NSTaskDidTerminateNotification" 
        object: task];
	[[[task standardOutput] fileHandleForReading] readInBackgroundAndNotify];
	[task launch];
	isConverting = YES;
}

- (void)stop
{
	isConverting = NO;
	[task terminate];
}

- (void)stopProcess:(id)stopTask
{
	[[NSNotificationCenter defaultCenter] removeObserver:self
		name:@"NSFileHandleReadCompletionNotification"
		object: [[stopTask standardOutput] fileHandleForReading]];
	[[NSNotificationCenter defaultCenter] removeObserver:self
		name:@"NSTaskDidTerminateNotification"
		object: stopTask];
	[stopTask release];
}

- (NSString *)currentFile
{
	return [convertFiles objectAtIndex: currentIndex];
}

- (void)taskDidEnd: (NSNotification *)aNotification
{
	int status = [[aNotification object] terminationStatus];
	[self stopProcess: [aNotification object]];
	if(status ==0) {
		currentIndex++;
		if(currentIndex > [convertFiles count]-1) {
			[delegate convertComplete:[convertFiles objectAtIndex: currentIndex-1]];
			isConverting = NO;
		}
		else {
			[delegate convertDone: [convertFiles objectAtIndex: currentIndex-1]];
			[self convert];
		}
	}
	else if(status == 15) {
		[delegate convertTerminated];
	}
}

- (void)getData: (NSNotification *)aNotification
{
    NSData *data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];
    if ([data length])
    {
		if(!memcmp([data bytes],"\r",1)) {
			NSArray *arr = [[[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease] componentsSeparatedByString:@" "];
			[delegate setPlayTime: [arr objectAtIndex:1]];
		}
	}
	
    [[aNotification object] readInBackgroundAndNotify];  
}

@end
